/*
 * Title:        CloudSim Toolkit
 * Description:  CloudSim (Cloud Simulation) Toolkit for Modeling and Simulation of Clouds
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 *
 * Copyright (c) 2009-2012, The University of Melbourne, Australia
 */

package org.cloudbus.cloudsim.CKP.datacenter;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.cloudbus.cloudsim.Log;
import org.cloudbus.cloudsim.CKP.datacenter.service.AppNetCloudlet;
import org.cloudbus.cloudsim.checkpoint.AppCheckpoint;
import org.cloudbus.cloudsim.checkpoint.CloudletCheckpoint;
import org.cloudbus.cloudsim.checkpoint.RecordToLogger;
import org.cloudbus.cloudsim.core.CloudSim;
import java.sql.*;
/**
 * This class represents an graph containing nodes and edges, used for input with an network-layer
 * Graphical-Output Restricions! EdgeColors: GraphicalProperties.getColorEdge NodeColors:
 * GraphicalProperties.getColorNode
 * 
 * @author Thomas Hohnstein
 * @since CloudSim Toolkit 1.0
 */
public class DCNetwork {

	protected List<DataCenterLink> linkList = null;

	protected List<DataCenterNode> nodeList = null;
	
	protected List<RootSwitch> rootSwitchList = null;
	
	protected List<AggregateSwitch> aggregateSwitchList = null;
	
	protected List<EdgeSwitch> edgeSwitchList = null;
	
	protected List<NetworkHost> hostList = null;
	
	protected NetworkStorageHost storageCenter = null;


	/**
	 * just the constructor to create an empty graph-object
	 */
	public DCNetwork() {
		linkList = new ArrayList<DataCenterLink>();
		nodeList = new ArrayList<DataCenterNode>();
		rootSwitchList = new ArrayList<RootSwitch>();
		aggregateSwitchList = new ArrayList<AggregateSwitch>();
		edgeSwitchList = new ArrayList<EdgeSwitch>();
		hostList = new ArrayList<NetworkHost>();
	}
	
	public List<DataCenterLink> getLinkList() {
		return linkList;
	}

	public void setLinkList(List<DataCenterLink> linkList) {
		this.linkList = linkList;
	}

	public List<DataCenterNode> getNodeList() {
		return nodeList;
	}

	public void setNodeList(List<DataCenterNode> nodeList) {
		this.nodeList = nodeList;
	}

	public List<RootSwitch> getRootSwitchList() {
		return rootSwitchList;
	}

	public void setRootSwitchList(List<RootSwitch> rootSwitchList) {
		this.rootSwitchList = rootSwitchList;
	}

	public List<AggregateSwitch> getAggregateSwitchList() {
		return aggregateSwitchList;
	}

	public void setAggregateSwitchList(List<AggregateSwitch> aggregateSwitchList) {
		this.aggregateSwitchList = aggregateSwitchList;
	}

	public List<EdgeSwitch> getEdgeSwitchList() {
		return edgeSwitchList;
	}

	public void setEdgeSwitchList(List<EdgeSwitch> edgeSwitchList) {
		this.edgeSwitchList = edgeSwitchList;
	}

	public List<NetworkHost> getHostList() {
		return hostList;
	}

	public void setHostList(List<NetworkHost> hostList) {
		this.hostList = hostList;
	}

	/**
	 * adds an link between two topological nodes
	 * 
	 * @param edge the topological link
	 */
	public void addLink(DataCenterLink edge) {
		linkList.add(edge);
	}

	/**
	 * adds an Topological Node to this graph
	 * 
	 * @param node the topological node to add
	 */
	public void addNode(DataCenterNode node) {
		nodeList.add(node);
	}
	

	/**
	 * returns the number of nodes contained inside the topological-graph
	 * 
	 * @return number of nodes
	 */
	public int getNumberOfNodes() {
		return nodeList.size();
	}

	/**
	 * returns the number of links contained inside the topological-graph
	 * 
	 * @return number of links
	 */
	public int getNumberOfLinks() {
		return linkList.size();
	}

	/**
	 * return an iterator through all network-graph links
	 * 
	 * @return the iterator throug all links
	 */
	public Iterator<DataCenterLink> getLinkIterator() {
		return linkList.iterator();
	}

	/**
	 * returns an iterator through all network-graph nodes
	 * 
	 * @return the iterator through all nodes
	 */
	public Iterator<DataCenterNode> getNodeIterator() {
		return nodeList.iterator();
	}
	

	public NetworkStorageHost getStorageCenter() {
		return storageCenter;
	}

	public void setStorageCenter(NetworkStorageHost storageCenter) {
		this.storageCenter = storageCenter;
	}
	
	public void clearVisited(){
		for(int i = 0; i < nodeList.size(); i++){
			nodeList.get(i).setHasVisited(false);
		}
	}

	/**
	 * prints out all internal node and link information
	 */
	@Override
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("topological-node-information: \n");

		for (DataCenterNode node : nodeList) {
			buffer.append(CloudSim.getEntityId(node.getName()) + " | x is: " + node.getWorldX() + " y is: "
					+ node.getWorldY() + "\n");
		}

		buffer.append("\n\n node-link-information:\n");

		for (DataCenterLink link : linkList) {
			buffer.append("from: " + link.getSrcNodeID() + " to: " + link.getDestNodeID() + " delay: "
					+ link.getLinkDelay() + "\n");
		}
		return buffer.toString();
	}
	
	public void printResult(int size,double totallength,double totaltime){
		double rootPacket = 0, aggPacket = 0, edgePacket = 0;
		
		for(int i = 0; i < this.rootSwitchList.size(); i++){
			rootPacket = rootPacket + rootSwitchList.get(i).hasReceviedCKPPacketSize
					+ rootSwitchList.get(i).hasReceviedServiceDataPacketSize;
		}
		
		for(int i = 0; i < this.aggregateSwitchList.size(); i++){
			aggPacket = aggPacket + aggregateSwitchList.get(i).hasReceviedCKPPacketSize
					+aggregateSwitchList.get(i).hasReceviedServiceDataPacketSize;
		}
		
		for(int i = 0; i < this.edgeSwitchList.size(); i++){
			edgePacket = edgePacket + edgeSwitchList.get(i).hasReceviedCKPPacketSize
					+ edgeSwitchList.get(i).hasReceviedServiceDataPacketSize;
		}
		

		RecordToLogger.logger.info("========== OUTPUT ==========");
		RecordToLogger.logger.info("RootSwitch Packet Processed: " + rootPacket);
		RecordToLogger.logger.info("AggregateSwitch Packet Processed: " + aggPacket);
		RecordToLogger.logger.info("EdgeSwitch Packet Processed: " + edgePacket);
		RecordToLogger.logger.info("========== ===== ==========");
		
	}

	public static void main(String[] args) {
		
		
		Timestamp nowdatatime = new Timestamp(System.currentTimeMillis()); 
		 try{
		      Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
		      String strurl="jdbc:odbc:driver={Microsoft Access Driver (*.mdb,*.accdb)};DBQ=c:\\db1.mdb";
		      System.out.println("ӳɹ!");
		      Connection con=DriverManager.getConnection(strurl) ;
		      System.out.println("ӳɹ!");
		      Statement stmt=con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,ResultSet.CONCUR_READ_ONLY);
		      
			  String sql="INSERT INTO log4j(runtime)  values("+"'" + nowdatatime+"'"+")";
			  stmt.execute(sql);
			  con.close();
		 
		 
		 
		 }
		
		     catch(Exception e)
		     { System.out.println(e); }

//		RecordToLogger.logger.info("INSERT INTO log4j(RCPP,ACPP,ECPP,RSPP,ASPP,ESPP,RPP,APP,EPP,ACS,CCS,TIME) values("+"'" + rootCKPPacket+"'"+","
//				
//				+"'" + aggCKPPacket+"'"+","+"'" + edgeCKPPacket+"'"+","+"'" + rootServicePacket+"'"+","+"'" + aggServicePacket+"'"+","+"'" + 
//				
//				edgeServicePacket+"'"+","+"'" + (rootCKPPacket+rootServicePacket)+"'"+","+"'" + (aggCKPPacket+aggServicePacket)+"'"+","+"'" + 
//				
//				(edgeCKPPacket+edgeServicePacket)+"'"+","+"'" + appStorageSize+"'"+","+"'" + cloudletStorageSize+"'"+","+"'" + nowdatatime+"'"+")");
	}
}
